import archr
import os
import logging
import unittest

test_location = os.path.dirname(os.path.realpath(__file__))


class TestangrUltimateTracer(unittest.TestCase):
    @unittest.skip("for now")  # Unless(archr._angr_available, "angr required")
    def test_dir_x86_64(self):
        target = archr.targets.LocalTarget(os.path.join(test_location, "../../binaries/tests/x86_64/dir_gcc_-O0"))
        dsb = archr.arsenal.DataScoutAnalyzer(target)
        apb = archr.arsenal.angrProjectAnalyzer(target, dsb)
        asb = archr.arsenal.angrStateAnalyzer(target, apb)
        utb = archr.arsenal.angrUltimateTracerAnalyzer(target, apb)

        import angr

        logging.getLogger("angr.bureau.bureau").setLevel(logging.DEBUG)

        _ = utb.make_project()
        trace = utb.fire(
            state=asb.fire(
                brk=None,
                args=["dir", "/"],
                add_options=angr.sim_options.unicorn | {angr.sim_options.ZERO_FILL_UNCONSTRAINED_MEMORY},
            )
        )
        # p = apb.fire()
        # s = asb.fire()
        # tech = trace.tracer_technique()
        # simgr = p.factory.simulation_manager(s)
        # simgr.use_technique(tech)
        # simgr.run()

        # assert len(simgr.traced) == 1


if __name__ == "__main__":
    unittest.main()



import socket

import docker.errors

import archr
import time
import unittest

from common import build_container


class TestDockerTargetSimple(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("cat")
        build_container("cat-stderr")
        build_container("entrypoint-true")
        build_container("entrypoint-false")
        build_container("crasher")
        build_container("nccat")
        build_container("entrypoint-env")
        build_container("entrypoint-sh-env")
        build_container("entrypoint-setarch-env")

    def test_cat(self):
        with archr.targets.DockerImageTarget("archr-test:cat").build().start() as t:
            p = t.run_command()
            p.stdin.write(b"Hello!\n")
            assert p.stdout.read(7) == b"Hello!\n"

    def test_cat_stderr(self):
        with archr.targets.DockerImageTarget("archr-test:cat-stderr").build().start() as t:
            p = t.run_command()
            p.stdin.write(b"Hello!\n")
            assert p.stderr.read(7) == b"Hello!\n"

    def test_entrypoint_true(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-true").build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == 0

    def test_entrypoint_false(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-false").build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == 1

    def test_entrypoint_crasher(self):
        with archr.targets.DockerImageTarget("archr-test:crasher").build().start() as t:
            p = t.run_command()
            p.wait()
            assert p.returncode == 139

    def test_entrypoint_env(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            p = t.run_command()
            stdout, _ = p.communicate()
            assert b"ARCHR=YES" in stdout.split(b"\n")

    def test_nccat_simple(self):
        with archr.targets.DockerImageTarget("archr-test:nccat").build().start() as t:
            t.run_command()
            assert t.tcp_ports == [1337]
            try:
                s = socket.create_connection((t.ipv4_address, 1337))
            except ConnectionRefusedError:
                time.sleep(5)
                s = socket.create_connection((t.ipv4_address, 1337))
            s.send(b"Hello\n")
            assert s.recv(6) == b"Hello\n"

    def test_context_env(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start() as t:
            with t.run_command() as p:
                stdout, _ = p.communicate()
            assert b"ARCHR=YES" in stdout.split(b"\n")

    def test_user(self):
        with archr.targets.DockerImageTarget("archr-test:entrypoint-env").build().start(user="nobody") as t:
            assert t.run_command(["touch", "/"]).wait() != 0
            assert t.run_command(["touch", "/"], user="root").wait() == 0

    def test_entrypoint_tricky(self):
        t = archr.targets.DockerImageTarget("archr-test:entrypoint-sh-env").build()
        assert t.target_args == ["/usr/bin/env", "YEAH"]
        t = archr.targets.DockerImageTarget("archr-test:entrypoint-setarch-env").build()
        assert t.target_args == ["/usr/bin/env", "HAHAHA"]

    def test_timeout(self):
        t = archr.targets.DockerImageTarget("archr-test:cat").build().start(timeout=3)
        import time

        time.sleep(8)
        # the target should be gone by now
        try:
            t.container.top()
            assert False, "The container did not exit after timeout."
        except docker.errors.NotFound:
            pass


if __name__ == "__main__":
    unittest.main()



from time import sleep

import archr
import unittest

from common import build_container

BIN_CAT = "/bin/cat"
CAT_ARGS = ["/etc/passwd"]
LTRACE_ARGS = "-f -e malloc+free+open+read+write+socket+bind+accept-@libc.so* -n 2".split()


class TestAnalyzerLtrace(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        build_container("socat")
        build_container("cat")

    def ltrace_proc(self, t, **kwargs):
        b = archr.analyzers.LTraceAnalyzer(t)
        with b.fire_context(trace_args=LTRACE_ARGS, **kwargs) as flight:
            sleep(1)
            flight.process.terminate()
        return flight.result

    def ltrace_attach(self, t, p, **kwargs):
        b = archr.analyzers.LTraceAttachAnalyzer(t)
        pid = p.pid if isinstance(t, archr.targets.LocalTarget) else t.get_proc_pid("socat")
        with b.fire_context(pid=pid, trace_args=LTRACE_ARGS, **kwargs) as flight:
            sleep(0.1)
            nc = flight.get_channel("tcp:0")  # misuse of flight
            nc.send(b"ahoi!")
            assert nc.readuntil(b"ahoi!", timeout=5) == b"ahoi!"

        return flight.result

    def check_ltrace_proc(self, t, **kwargs):
        output = self.ltrace_proc(t, **kwargs)
        assert b"cat->open" in output
        assert b"cat->malloc" in output
        assert b"cat->read" in output

    def check_ltrace_attach(self, t, **kwargs):
        target = t.run_command()  # start target
        output = self.ltrace_attach(t, target, **kwargs)
        target.terminate()
        assert b"exe->accept" in output
        assert b"exe->malloc" in output
        assert b"exe->free" in output
        assert b"exe->read" in output
        assert b"exe->write" in output

    @unittest.skip("broken")
    def test_ltrace_proc_local(self):
        with archr.targets.LocalTarget(["/bin/cat", "/etc/passwd"]).build().start() as t:
            self.check_ltrace_proc(t)

    @unittest.skip("broken")
    def test_ltrace_proc_docker(self):
        with archr.targets.DockerImageTarget(
            "archr-test:cat", target_args=["/bin/cat", "/etc/passwd"]
        ).build().start() as t:
            self.check_ltrace_proc(t)

    @unittest.skip("broken")
    def test_ltrace_attach_local(self):
        with archr.targets.LocalTarget(
            "socat tcp-l:7573,reuseaddr exec:cat".split(), tcp_ports=[7573]
        ).build().start() as t:
            self.check_ltrace_attach(t)

    @unittest.skip("broken")
    def test_ltrace_attach_docker(self):
        with archr.targets.DockerImageTarget("archr-test:socat").build().start() as t:
            self.check_ltrace_attach(t)


if __name__ == "__main__":
    unittest.main()
